#ifndef COMSTATISTICS_H
#define COMSTATISTICS_H

#include <QString>

class ComStatistics
{
public:
    ComStatistics(QString sensor_name);
    void ReQuestCount_increase();
    void CrcErrorCount_increase();
    void TimeoutCount_increase();
    void Update_Recvtime(int ms);
    void Save2File(QString filepath);

private:
    int request_count;
    int crcerror_count;
    int timeout_count;

    int max_recvtime;  //ms
    int min_recvtime;  //ms
    int total_recvtime;  //ms

    QString tag_name;
};

#endif // COMSTATISTICS_H
